/* Copyright 2021 Thomas Schneider
 *
 * This file is a part of Fedilab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * Fedilab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with Fedilab; if not,
 * see <http://www.gnu.org/licenses>. */
package app.fedilab.android.asynctasks;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;

import app.fedilab.android.client.API;
import app.fedilab.android.client.APIResponse;
import app.fedilab.android.client.Entities.Account;
import app.fedilab.android.interfaces.OnPostSubscription;


public class PostSubscriptionAsyncTask {

    private final OnPostSubscription listener;
    private final WeakReference<Context> contextReference;
    private APIResponse apiResponse;
    private final String endpoint;
    private final Account account;

    public PostSubscriptionAsyncTask(Context context, Account account, String endpoint, OnPostSubscription onPostSubscription) {
        this.contextReference = new WeakReference<>(context);
        this.listener = onPostSubscription;
        this.endpoint = endpoint;
        this.account = account;
        doInBackground();
    }


    protected void doInBackground() {
        new Thread(() -> {
            apiResponse = new API(contextReference.get(), account.getInstance(), account.getToken()).getPushSubscription();
            if (apiResponse == null || apiResponse.getPushSubscription() == null || endpoint.compareTo(apiResponse.getPushSubscription().getEndpoint()) != 0) {
                apiResponse = new API(contextReference.get(), account.getInstance(), account.getToken()).pushSubscription(endpoint, account);
            }
            //TODO: remove this line
            apiResponse = new API(contextReference.get(), account.getInstance(), account.getToken()).pushSubscription(endpoint, account);
            List<Account> accountList = new ArrayList<>();
            accountList.add(account);
            apiResponse.setAccounts(accountList);
            Handler mainHandler = new Handler(Looper.getMainLooper());
            Runnable myRunnable = () -> listener.onSubscription(apiResponse);
            mainHandler.post(myRunnable);
        }).start();
    }

}
